'''Information about supported locales, countries, and languages.'''

# this is part of langpack-o-matic, by Martin Pitt <martin.pitt@canonical.com>
# (C) 2005 Canonical Ltd.

import os.path

class SupportedLocales:
    def __init__(self):
        self.languages = {}
        mapdir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 
            '..', 'maps')
        for line in open(os.path.join(mapdir, 'languages')):
            if line.find('#') == -1:
                (code, name) = line.strip().split(':')
                self.languages[code] = name

        self.countries = {}
        for line in open(os.path.join(mapdir, 'countries')):
            if line.find('#') == -1 and line.find(':') >= 0:
                (code, name) = line.strip().split(':')
                self.countries[code] = name

    def known_language(self, lang):
        '''Return whether the given language is supported.

        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.known_language('en')
        True
        >>> sl.known_language('de')
        True
        >>> sl.known_language('xx')
        False
        '''

        return self.languages.has_key(lang)

    def known_country(self, country):
        '''Return whether the given language is supported.
        
        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.known_country('GB')
        True
        >>> sl.known_country('US')
        True
        >>> sl.known_country('DE')
        True
        >>> sl.known_country('EN')
        False
        '''

        return self.countries.has_key(country)

    def known_locale(self, locale):
        '''Return whether the given locale is supported.
        
        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.known_locale('en_GB.UTF-8')
        True
        >>> sl.known_locale('de_DE')
        True
        >>> sl.known_locale('xx_XX')
        False
        '''

        # split off language and just check that
        locale = (locale.split('@')[0]).split('.')[0]
        return self.languages.has_key(locale.split('_')[0])

    def language_name(self, code):
        '''Return the name of the given language code.
        
        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.language_name('de')
        'German'
        >>> sl.language_name('en')
        'English'
        >>> sl.language_name('xx')
        Traceback (most recent call last):
        KeyError: 'xx'
        '''

        return self.languages[code]

    def country_name(self, code):
        '''Return the name of the given country code.

        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.country_name('GB')
        'United Kingdom of Great Britain and Northern Ireland'
        >>> sl.country_name('US')
        'United States of America'
        >>> sl.country_name('DE')
        'Germany'
        >>> sl.country_name('EN')
        Traceback (most recent call last):
        KeyError: 'EN'
        '''

        return self.countries[code]

# unit test
if __name__ == '__main__':
    import doctest
    doctest.testmod()
